searchData={"items":[{"type":"module","title":"NervesKey.PKCS11","doc":"This module contains helper methods for loading and using the PKCS #11\nmodule for NervesKey in Elixir. You don't need to use these methods to\nuse the shared library.","ref":"NervesKey.PKCS11.html"},{"type":"function","title":"NervesKey.PKCS11.load_engine/0","doc":"Load the OpenSSL engine","ref":"NervesKey.PKCS11.html#load_engine/0"},{"type":"function","title":"NervesKey.PKCS11.private_key/2","doc":"Return the key map for passing a private key to ssl_opts.\n\nThis method creates the key map that the `:crypto` library can\nuse to properly route private key operations to the PKCS #11\nshared library.\n\nOptions:\n\n* `:i2c` - which I2C bus (defaults to I2C bus 0 (`/dev/i2c-0`))\n* `:type` - :nerves_key or :trust_and_go (defaults to :nerves_key)\n* `:certificate` - which certificate on the NervesKey to use (defaults to `:primary`)\n\nPassing `{:i2c, 1}` is still supported, but should be updated to use keyword\nlist form for the options.","ref":"NervesKey.PKCS11.html#private_key/2"},{"type":"type","title":"NervesKey.PKCS11.certificate_pair/0","doc":"The device/signer certificate pair to use","ref":"NervesKey.PKCS11.html#t:certificate_pair/0"},{"type":"type","title":"NervesKey.PKCS11.i2c_bus/0","doc":"I2C bus","ref":"NervesKey.PKCS11.html#t:i2c_bus/0"},{"type":"type","title":"NervesKey.PKCS11.option/0","doc":"Option for which NervesKey and certificate to use.\n\n* `:i2c` - which I2C bus\n* `:certificate` - which NervesKey certificate to use (`:primary` or `:aux`)\n* `:type` - if using pre-provisioned ATECC608B Trust and Go parts, specify `:trust_and_go`","ref":"NervesKey.PKCS11.html#t:option/0"},{"type":"extras","title":"NervesKey.PKCS11","doc":"# NervesKey.PKCS11\n\n[![CircleCI](https://circleci.com/gh/nerves-hub/nerves_key_pkcs11.svg?style=svg)](https://circleci.com/gh/nerves-hub/nerves_key_pkcs11)\n[![Hex version](https://img.shields.io/hexpm/v/nerves_key_pkcs11.svg \"Hex version\")](https://hex.pm/packages/nerves_key_pkcs11)\n\nThis is a minimal implementation of [PKCS #11](https://en.wikipedia.org/wiki/PKCS_11)\nfor using the NervesKey with OpenSSL and other programs.  The NervesKey is a specific\nconfiguration of the ATECC508A/ATECC608A chips that holds one private key in slot 0. If\nyou're using this chip in a similar configuration, this should work for you as well.\n\nSupported features:\n\n* ECDSA\n\nThis library is organized to make it easy to integrate into Elixir and is\nwritten with an expectation that provisioning, extracting certificates, etc. is done\nvia other means (like using [nerves_key](https://hex.pm/nerves_key).) If you're not\nusing Elixir, you can still run `make` and copy `priv/nerves_key_pkcs11.so` to\na conveniently location. Elixir isn't needed to build the C library.\n\nAnother option is to look at\n[cryptoauth-openssl-engine](https://github.com/MicrochipTech/cryptoauth-openssl-engine)\nor [cryptoauthlib](https://github.com/MicrochipTech/cryptoauthlib).","ref":"readme.html"},{"type":"extras","title":"Building - NervesKey.PKCS11","doc":"This library is self-contained with no compile-time dependencies. At runtime, it\nisn't useful unless you have another program around that uses a PKCS #11 shared\nlibrary.\n\nIf using it with Elixir, add a dependency to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:nerves_key_pkcs11, \"~> 1.0\"}\n  ]\nend\n```\n\nIf not using Elixir, run `make`. You may need to set $(CC) or $(CFLAGS) if you're\ncrosscompiling.","ref":"readme.html#building"},{"type":"extras","title":"Slot definition - NervesKey.PKCS11","doc":"PKCS #11 uses the term slot to refer to cryptographic devices. This library can\nuse either slot ID (if called directly) or the slot's token ID (if called via\nlibp11) to find the NervesKey. Various parameters are mapped into the slot ID\naccording to the table below:\n\nSlot range  | I2C bus   | Bus address              | Certificate\n------------|-----------|--------------------------|------------\n0-15        | slot      | 0x60 (ATECC default)     | Primary or auxiliary\n16-31       | slot-16   | 0x35 (Trust&Go versions) | Primary or auxiliary\n\nOn Linux, the I2C bus number in the table above determines the device file. For\nexample, a bus number of \"1\" maps to `/dev/i2c-1`. NervesKey devices support a\nprimary and auxiliary set of certificates. Normally only the primary device\ncertificate is used. Under some conditions, it's useful to write a second set of\ncertificates to the device and those can be referenced by using the \"Auxiliary\"\nrows in the table.  This library currently need to differentiate between primary\nand auxiliary certificates so that's not represented in the slot ID. It may be\nin the future.\n\nThe [PKCS #11 URI](https://tools.ietf.org/html/rfc7512) for addressing the\ndesired NervesKey has the form:\n\n```text\npkcs11:token=1\n```","ref":"readme.html#slot-definition"},{"type":"extras","title":"OpenSSL integration - NervesKey.PKCS11","doc":"To use this with OpenSSL, you'll need `libpkcs11.so`. This library comes from\n[OpenSC's libp11](https://github.com/OpenSC/libp11) and can be installed on\nDebian systems by running:\n\n```sh\nsudo apt install libengine-pkcs11-openssl1.1\n```","ref":"readme.html#openssl-integration"},{"type":"extras","title":"Invocation from Erlang and Elixir - NervesKey.PKCS11","doc":"Erlang's `crypto` application provides an API for loading OpenSSL engines. See\n[the Erlang crypto User's Guide](http://erlang.org/doc/apps/crypto/engine_load.html)\nfor details on this feature. `NervesKey.PKCS11.load_engine/0` is a helper method\nto make the `:crypto.engine_load/3` call for you. It uses OpenSSL's `dynamic`\nengine to load `libpkcs11.so` which in turn loads this PKCS #11 implementation.\nHere's an example call in Elixir:\n\n```elixir\n{:ok, engine} = NervesKey.PKCS11.load_engine()\n```\n\nIf this doesn't work, you'll likely have to look at the implementation of\n`load_engine/0` and fine tune the shared library paths or control commands.\n\nAfter you load the engine, you'll eventually want to use it. The intended use\ncase is for delegate the ECDSA operation to the ATECC508A for use with TLS\nconnections. You'll need to obtain the X.509 certificate that corresponds to the\nprivate key held in the ATECC508A through some mechanism. Then in your SSL\noptions, you'll have something like this:\n\n```elixir\n[\n  key: NervesKey.PKCS11.private_key(engine, i2c: 1, type: :nerves_key),\n  certfile: \"device-cert.pem\",\n]\n```\n\nThe `NervesKey.PKCS11.private_key/2` helper method will create the appropriate\nmap so that Erlang's `:crypto` library can properly call into OpenSSL.\n\nIf you are using a pre-provisioned ATECC608B or similar that's labelled a Trust\nand Go part, specify `:trust_and_go` for the `:type`.","ref":"readme.html#invocation-from-erlang-and-elixir"},{"type":"extras","title":"Sharing the NervesKey - NervesKey.PKCS11","doc":"If you have other code using the NervesKey, it might conflict with this library. There's\nno lock file or mechanism to keep more than one process from accessing the ATECC508A\nchip simultaneously. This is not expected to be an issue at runtime since the main\nreason to access the NervesKey in another process is to provision it and that's not\nsomething one would do when trying to use this library to assist a TLS negotiation.","ref":"readme.html#sharing-the-nerveskey"},{"type":"extras","title":"License - NervesKey.PKCS11","doc":"The Elixir and most C code is licensed under the 2-Clause BSD License.\n\nThe header file for the PKCS #11 function prototypes and structures, `pkcs11.h`,\nhas the following license:\n\n```text\n/* pkcs11.h\n   Copyright 2006, 2007 g10 Code GmbH\n   Copyright 2006 Andreas Jellinghaus\n\n   This file is free software; as a special exception the author gives\n   unlimited permission to copy and/or distribute it, with or without\n   modifications, as long as this notice is preserved.\n\n   This file is distributed in the hope that it will be useful, but\n   WITHOUT ANY WARRANTY, to the extent permitted by law; without even\n   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR\n   PURPOSE.  */\n```","ref":"readme.html#license"}],"content_type":"text/markdown"}